@@ -1,7 +1,7 @@
-# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf8 -*-
+# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf-8 -*-
 #
-# Copyright 2002 Ben Escoto <ben@emerose.org>
-# Copyright 2007 Kenneth Loafman <kenneth@loafman.com>
+# Copyright 2002 Ben Escoto
+# Copyright 2007 Kenneth Loafman
 #
 # This file is part of duplicity.
 #
@@ -22,22 +22,26 @@
 import os
 
 import duplicity.backend
-from duplicity import path, progress
+from duplicity import (
+    path,
+    progress,
+)
 from duplicity.errors import BackendException
 
 
 class LocalBackend(duplicity.backend.Backend):
-    u"""Use this backend when saving to local disk
+    """Use this backend when saving to local disk
 
     Urls look like file://testfiles/output.  Relative to root can be
     gotten with extra slash (file:///usr/local).
 
     """
+
     def __init__(self, parsed_url):
         duplicity.backend.Backend.__init__(self, parsed_url)
         # The URL form "file:MyFile" is not a valid duplicity target.
-        if not parsed_url.path.startswith(u'//'):
-            raise BackendException(u"Bad file:// path syntax.")
+        if not parsed_url.path.startswith("//"):
+            raise BackendException("Bad file:// path syntax.")
         self.remote_pathdir = path.Path(parsed_url.path[2:])
         try:
             os.makedirs(self.remote_pathdir.base)
@@ -57,12 +61,12 @@ class LocalBackend(duplicity.backend.Backend):
         source_path.setdata()
         source_size = source_path.getsize()
         progress.report_transfer(0, source_size)
-        target_path.writefileobj(source_path.open(u"rb"))
+        target_path.writefileobj(source_path.open("rb"))
         progress.report_transfer(source_size, source_size)
 
     def _get(self, filename, local_path):
         source_path = self.remote_pathdir.append(filename)
-        local_path.writefileobj(source_path.open(u"rb"))
+        local_path.writefileobj(source_path.open("rb"))
 
     def _list(self):
         return self.remote_pathdir.listdir()
@@ -70,11 +74,15 @@ class LocalBackend(duplicity.backend.Backend):
     def _delete(self, filename):
         self.remote_pathdir.append(filename).delete()
 
+    def _delete_list(self, filenames):
+        for filename in filenames:
+            self.remote_pathdir.append(filename).delete()
+
     def _query(self, filename):
         target_file = self.remote_pathdir.append(filename)
         target_file.setdata()
         size = target_file.getsize() if target_file.exists() else -1
-        return {u'size': size}
+        return {"size": size}
 
 
-duplicity.backend.register_backend(u"file", LocalBackend)
+duplicity.backend.register_backend("file", LocalBackend)
