@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf-8 -*-
 #
-# Copyright 2022 Kenneth Loafman <kenneth@loafman.com>
+# Copyright 2022 Kenneth Loafman
 #
 # This file is part of duplicity.
 #
@@ -113,8 +113,8 @@ OptionKwargs = dict(
         default=dflt(config.async_concurrency),
     ),
     azure_blob_tier=dict(
-        metavar=_("Hot|Cool|Archive"),
-        help="Standard storage tier used for storing backup files (Hot|Cool|Archive)",
+        metavar=_("Hot|Cool|Cold|Archive"),
+        help="Standard storage tier used for storing backup files (Hot|Cool|Cold|Archive)",
         default=dflt(config.azure_blob_tier),
     ),
     azure_max_connections=dict(
@@ -349,8 +349,8 @@ OptionKwargs = dict(
         default=dflt(config.ftp_connection),
     ),
     full_if_older_than=dict(
-        metavar=_("interval"),
-        type=check_interval,
+        metavar=_("time"),
+        type=check_time_delta,
         help="Perform full backup if last full is older than 'time'",
         default=dflt(config.full_if_older_than),
     ),
@@ -371,6 +371,12 @@ OptionKwargs = dict(
         ),
         default=dflt(config.gpg_options),
     ),
+    use_gpgsm=dict(
+        action="store_true",
+        dest="use_gpgsm",
+        help="Use gpgsm tool, which is very similar gpg but using PKCS#7 file format.",
+        default=dflt(config.use_gpgsm),
+    ),
     hidden_encrypt_key=dict(
         metavar=_("gpg-key-id"),
         type=set_hidden_encrypt_key,
