@@ -1,6 +1,6 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf-8 -*-
 #
-# Copyright 2022 Kenneth Loafman <kenneth@loafman.com>
+# Copyright 2022 Kenneth Loafman
 #
 # This file is part of duplicity.
 #
@@ -115,7 +115,7 @@ def parse_log_options(arglist):
     try:
         args, remainder = parser.parse_known_intermixed_args(arglist)
     except (argparse.ArgumentError, argparse.ArgumentTypeError) as e:
-        raise CommandLineError(str(e))
+        command_line_error(str(e))
 
     return args, remainder
 
@@ -140,7 +140,7 @@ def parse_cmdline_options(arglist):
     try:
         args, remainder = parser.parse_known_intermixed_args(remainder)
     except (argparse.ArgumentError, argparse.ArgumentTypeError) as e:
-        raise CommandLineError(str(e))
+        command_line_error(str(e))
 
     # let's test the command and try to assume which action,
     # eventually err out if no valid action could be determined/was given
@@ -261,9 +261,15 @@ def process_command_line(cmdline_list):
             hidden_recipients=src.hidden_recipients,
         )
     else:
-        config.gpg_binary = util.which("gpg")
+        if config.use_gpgsm:
+            program = "gpgsm"
+        else:
+            program = "gpg"
+        config.gpg_binary = util.which(program)
     gpg_version = ".".join(map(str, config.gpg_profile.gpg_version))
     log.Info(_(f"GPG binary is {config.gpg_binary}, version {gpg_version}"))
+    if config.use_gpgsm and config.gpg_profile.gpg_version < (2, 2, 27):
+        log.FatalError(f"Version {gpg_version} of gpgsm is not supported.  Minimum version is 2.2.27")
 
     # --use-agent is not safe for symmetric encryption.
     # Notify user and let them decide.
