@@ -1,7 +1,7 @@
 # -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf-8 -*-
 #
-# Copyright 2002 Ben Escoto <ben@emerose.org>
-# Copyright 2007 Kenneth Loafman <kenneth@loafman.com>
+# Copyright 2002 Ben Escoto
+# Copyright 2007 Kenneth Loafman
 #
 # This file is part of duplicity.
 #
@@ -19,52 +19,50 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
-u"""
+"""
 duplicity's gpg interface, builds upon Frank Tobin's GnuPGInterface
 which is now patched with some code for iterative threaded execution
 see duplicity's README for details
 """
 
-from builtins import next
-from builtins import str
-from builtins import object
-import os
-import sys
-import tempfile
-import re
 import gzip
 import locale
+import os
+import re
+import sys
+import tempfile
 
 from duplicity import config
 from duplicity import gpginterface
+from duplicity import log
 from duplicity import tempdir
 from duplicity import util
-from duplicity.gpg_error_codes import gpg_error_codes
 
-try:
-    from hashlib import sha1
-    from hashlib import md5
-except ImportError:
-    from sha import new as sha1
-    from md5 import new as md5
+from hashlib import sha1
+from hashlib import md5
 
 blocksize = 256 * 1024
 
 
 class GPGError(Exception):
-    u"""
+    """
     Indicate some GPG Error
     """
+
     pass
 
 
 class GPGProfile(object):
-    u"""
+    """
     Just hold some GPG settings, avoid passing tons of arguments
     """
-    def __init__(self, passphrase=None, sign_key=None,
-                 recipients=None, hidden_recipients=None):
-        u"""
+
+    _version_re = re.compile(
+        b"^gpg.*\\(GnuPG(?:/MacGPG2)?\\) (?P<maj>[0-9]+)\\.(?P<min>[0-9]+)\\.(?P<bug>[0-9]+)(-.+)?$"
+    )
+
+    def __init__(self, passphrase=None, sign_key=None, recipients=None, hidden_recipients=None):
+        """
         Set all data with initializer
 
         passphrase is the passphrase.  If it is None (not ""), assume
@@ -72,7 +70,7 @@ class GPGProfile(object):
         indicated, and recipients should be a list of keys.  For all
         keys, the format should be an hex key like 'AA0E73D2'.
         """
-        assert passphrase is None or isinstance(passphrase, (str, u"".__class__))
+        assert passphrase is None or isinstance(passphrase, str)
 
         self.passphrase = passphrase
         self.signing_passphrase = passphrase
@@ -92,9 +90,6 @@ class GPGProfile(object):
 
         self.gpg_version = self.get_gpg_version(config.gpg_binary)
 
-    rc = re.compile
-    _version_re = rc(b'^gpg.*\\(GnuPG(?:/MacGPG2)?\\) (?P<maj>[0-9]+)\\.(?P<min>[0-9]+)\\.(?P<bug>[0-9]+)(-.+)?$')
-
     def get_gpg_version(self, binary):
         gnupg = gpginterface.GnuPG()
         if binary is not None:
@@ -106,20 +101,21 @@ class GPGProfile(object):
                 gnupg.options.extra_args.append(opt)
 
         # get gpg version
-        res = gnupg.run([u"--version"], create_fhs=[u"stdout"])
-        line = res.handles[u"stdout"].readline().rstrip()
+        res = gnupg.run(["--version"], create_fhs=["stdout"])
+        line = res.handles["stdout"].readline().rstrip()
         m = self._version_re.search(line)
         if m is not None:
-            return int(m.group(u"maj")), int(m.group(u"min")), int(m.group(u"bug"))
-        raise GPGError(u"failed to determine gnupg version of %s from %s" % (binary, line))
+            return int(m.group("maj")), int(m.group("min")), int(m.group("bug"))
+        raise GPGError(f"failed to determine gnupg version of {binary} from {line}")
 
 
 class GPGFile(object):
-    u"""
+    """
     File-like object that encrypts decrypts another file on the fly
     """
+
     def __init__(self, encrypt, encrypt_path, profile):
-        u"""
+        """
         GPGFile initializer
 
         If recipients is set, use public key encryption and encrypt to
@@ -144,14 +140,15 @@ class GPGFile(object):
         if config.gpg_binary is not None:
             gnupg.call = config.gpg_binary
         gnupg.options.meta_interactive = 0
-        gnupg.options.extra_args.append(u'--no-secmem-warning')
-        gnupg.options.extra_args.append(u'--ignore-mdc-error')
+        gnupg.options.extra_args.append("--no-secmem-warning")
+        if not config.use_gpgsm:
+            gnupg.options.extra_args.append("--ignore-mdc-error")
 
         # Support three versions of gpg present 1.x, 2.0.x, 2.1.x
         if profile.gpg_version[:1] == (1,):
             if config.use_agent:
                 # gpg1 agent use is optional
-                gnupg.options.extra_args.append(u'--use-agent')
+                gnupg.options.extra_args.append("--use-agent")
 
         elif profile.gpg_version[:2] == (2, 0):
             pass
@@ -160,10 +157,10 @@ class GPGFile(object):
             if not config.use_agent:
                 # This forces gpg2 to ignore the agent.
                 # Necessary to enforce truly non-interactive operation.
-                gnupg.options.extra_args.append(u'--pinentry-mode=loopback')
+                gnupg.options.extra_args.append("--pinentry-mode=loopback")
 
         else:
-            raise GPGError(u"Unsupported GNUPG version, %s" % profile.gpg_version)
+            raise GPGError(f"Unsupported GNUPG version, {profile.gpg_version}")
 
         # user supplied options
         if config.gpg_options:
@@ -173,7 +170,7 @@ class GPGFile(object):
         cmdlist = []
         if profile.sign_key:
             gnupg.options.default_key = profile.sign_key
-            cmdlist.append(u"--sign")
+            cmdlist.append("--sign")
         # encrypt: sign key needs passphrase
         # decrypt: encrypt key needs passphrase
         # special case: allow different symmetric pass with empty sign pass
@@ -184,53 +181,68 @@ class GPGFile(object):
         # in case the passphrase is not set, pass an empty one to prevent
         # TypeError: expected a character buffer object on .write()
         if passphrase is None:
-            passphrase = u""
+            passphrase = ""
 
         if encrypt:
             if profile.recipients:
                 gnupg.options.recipients = profile.recipients
-                cmdlist.append(u'--encrypt')
             if profile.hidden_recipients:
                 gnupg.options.hidden_recipients = profile.hidden_recipients
-                cmdlist.append(u'--encrypt')
-            if not (profile.recipients or profile.hidden_recipients):
-                cmdlist.append(u'--symmetric')
+            if profile.recipients or profile.hidden_recipients:
+                cmdlist.append("--encrypt")
+            else:
+                cmdlist.append("--symmetric")
+            if not config.use_gpgsm:
                 # use integrity protection
-                gnupg.options.extra_args.append(u'--force-mdc')
+                gnupg.options.extra_args.append("--force-mdc")
             # Skip the passphrase if using the agent
             if config.use_agent:
-                gnupg_fhs = [u'stdin', ]
+                gnupg_fhs = [
+                    "stdin",
+                ]
             else:
-                gnupg_fhs = [u'stdin', u'passphrase']
-            p1 = gnupg.run(cmdlist,
-                           create_fhs=gnupg_fhs,
-                           attach_fhs={u'stdout': encrypt_path.open(u"wb"),
-                                       u'stderr': self.stderr_fp})
+                gnupg_fhs = [
+                    "stdin",
+                    "passphrase",
+                ]
+            # Turn off compression if needed
+            if not config.compression:
+                cmdlist.append("--compress-algo=none")
+            p1 = gnupg.run(
+                cmdlist,
+                create_fhs=gnupg_fhs,
+                attach_fhs={
+                    "stdout": encrypt_path.open("wb"),
+                    "stderr": self.stderr_fp,
+                },
+            )
             if not config.use_agent:
-                p1.handles[u'passphrase'].write(passphrase)
-                p1.handles[u'passphrase'].close()
-            self.gpg_input = p1.handles[u'stdin']
+                p1.handles["passphrase"].write(passphrase)
+                p1.handles["passphrase"].close()
+            self.gpg_input = p1.handles["stdin"]
         else:
             if (profile.recipients or profile.hidden_recipients) and profile.encrypt_secring:
-                cmdlist.append(u'--secret-keyring')
+                cmdlist.append("--secret-keyring")
                 cmdlist.append(profile.encrypt_secring)
-            gpg_attach = {u'stdin': encrypt_path.open(u"rb"),
-                          u'stderr': self.stderr_fp}
+            gpg_attach = {"stdin": encrypt_path.open("rb"), "stderr": self.stderr_fp}
             if profile.sign_key:
                 self.status_fp = tempfile.TemporaryFile(dir=tempdir.default().dir())
-                gpg_attach[u'status'] = self.status_fp
+                gpg_attach["status"] = self.status_fp
             # Skip the passphrase if using the agent
             if config.use_agent:
-                gnupg_fhs = [u'stdout', ]
+                gnupg_fhs = [
+                    "stdout",
+                ]
             else:
-                gnupg_fhs = [u'stdout', u'passphrase']
-            p1 = gnupg.run([u'--decrypt'],
-                           create_fhs=gnupg_fhs,
-                           attach_fhs=gpg_attach)
+                gnupg_fhs = [
+                    "stdout",
+                    "passphrase",
+                ]
+            p1 = gnupg.run(["--decrypt"], create_fhs=gnupg_fhs, attach_fhs=gpg_attach)
             if not config.use_agent:
-                p1.handles[u'passphrase'].write(passphrase)
-                p1.handles[u'passphrase'].close()
-            self.gpg_output = p1.handles[u'stdout']
+                p1.handles["passphrase"].write(passphrase)
+                p1.handles["passphrase"].close()
+            self.gpg_output = p1.handles["stdout"]
         self.gpg_process = p1
         self.encrypt = encrypt
 
@@ -247,10 +259,7 @@ class GPGFile(object):
         try:
             res = self.gpg_input.write(buf)
             if res is not None:
-                if sys.version_info.major >= 3:
-                    self.byte_count += res
-                else:
-                    self.byte_count += len(res)
+                self.byte_count += res
         except Exception:
             self.gpg_failed()
         return res
@@ -260,26 +269,24 @@ class GPGFile(object):
 
     def seek(self, offset):
         assert not self.encrypt
-        assert offset >= self.byte_count, u"%d < %d" % (offset, self.byte_count)
+        assert offset >= self.byte_count, f"{int(offset)} < {int(self.byte_count)}"
         if offset > self.byte_count:
             self.read(offset - self.byte_count)
 
     def gpg_failed(self):
-        ret = self.gpg_process.returned >> 8
-        msg = gpg_error_codes.get(ret, u"GPG returned an unknown error code: %d" % ret)
-        if ret != 2:
-            msg += u"GPG Failed, see log below:\n"
-            msg += u"===== Begin GnuPG log =====\n"
-            self.stderr_fp.seek(0)
-            for line in self.stderr_fp:
-                try:
-                    msg += str(line.strip(), locale.getpreferredencoding(), u'replace') + u"\n"
-                except Exception as e:
-                    msg += line.strip() + u"\n"
-            msg += u"===== End GnuPG log =====\n"
+        msg = "GPG Failed, see log below:\n"
+        msg += "===== Begin GnuPG log =====\n"
+        self.stderr_fp.seek(0)
+        for line in self.stderr_fp:
+            try:
+                msg += f"{str(line.strip(), locale.getpreferredencoding(), u'replace')}\n"
+            except Exception as e:
+                msg += f"{line.strip()}\n"
+        msg += "===== End GnuPG log =====\n"
+        if not (msg.find("invalid packet (ctb=14)") > -1):
             raise GPGError(msg)
         else:
-            return u""
+            return ""
 
     def close(self):
         if self.encrypt:
@@ -315,7 +322,7 @@ class GPGFile(object):
         self.closed = 1
 
     def set_signature(self):
-        u"""
+        """
         Set self.signature to signature keyID
 
         This only applies to decrypted files.  If the file was not
@@ -323,8 +330,7 @@ class GPGFile(object):
         """
         self.status_fp.seek(0)
         status_buf = self.status_fp.read()
-        match = re.search(b"^\\[GNUPG:\\] GOODSIG ([0-9A-F]*)",
-                          status_buf, re.M)
+        match = re.search(b"GOODSIG ([0-9A-F]*)", status_buf, re.M)
         if not match:
             self.signature = None
         else:
@@ -332,17 +338,15 @@ class GPGFile(object):
             self.signature = match.group(1).decode()
 
     def get_signature(self):
-        u"""
+        """
         Return  keyID of signature, or None if none
         """
         assert self.closed
         return self.signature
 
 
-def GPGWriteFile(block_iter, filename, profile,
-                 size=200 * 1024 * 1024,
-                 max_footer_size=16 * 1024):
-    u"""
+def GPGWriteFile(block_iter, filename, profile, size=200 * 1024 * 1024, max_footer_size=16 * 1024):
+    """
     Write GPG compressed file of given size
 
     This function writes a gpg compressed file by reading from the
@@ -366,14 +370,14 @@ def GPGWriteFile(block_iter, filename, profile,
     from duplicity import path
 
     def top_off(bytelen, file):
-        u"""
+        """
         Add bytelen of incompressible data to to_gpg_fp
 
         In this case we take the incompressible data from the
         beginning of filename (it should contain enough because size
         >> largest block size).
         """
-        incompressible_fp = open(filename, u"rb")
+        incompressible_fp = open(filename, "rb")
         assert util.copyfileobj(incompressible_fp, file.gpg_input, bytelen) == bytelen
         incompressible_fp.close()
 
@@ -394,6 +398,8 @@ def GPGWriteFile(block_iter, filename, profile,
             except StopIteration:
                 at_end_of_blockiter = 1
                 break
+            except Exception as e:
+                log.FatalError(f"Read error on {os.fsdecode(filename)}: {str(e)}")
             file.write(data)
 
         file.write(block_iter.get_footer())
@@ -411,7 +417,7 @@ def GPGWriteFile(block_iter, filename, profile,
 
 
 def GzipWriteFile(block_iter, filename, size=200 * 1024 * 1024, gzipped=True):
-    u"""
+    """
     Write gzipped compressed file of given size
 
     This is like the earlier GPGWriteFile except it writes a gzipped
@@ -422,10 +428,12 @@ def GzipWriteFile(block_iter, filename, size=200 * 1024 * 1024, gzipped=True):
     The input requirements on block_iter and the output is the same as
     GPGWriteFile (returns true if wrote until end of block_iter).
     """
+
     class FileCounted(object):
-        u"""
+        """
         Wrapper around file object that counts number of bytes written
         """
+
         def __init__(self, fileobj):
             self.fileobj = fileobj
             self.byte_count = 0
@@ -438,11 +446,11 @@ def GzipWriteFile(block_iter, filename, size=200 * 1024 * 1024, gzipped=True):
         def close(self):
             return self.fileobj.close()
 
-    file_counted = FileCounted(open(filename, u"wb"))
+    file_counted = FileCounted(open(filename, "wb"))
 
     # if gzipped wrap with GzipFile else plain file out
     if gzipped:
-        outfile = gzip.GzipFile(None, u"wb", 6, file_counted)
+        outfile = gzip.GzipFile(None, "wb", 6, file_counted)
     else:
         outfile = file_counted
     at_end_of_blockiter = 0
@@ -462,7 +470,7 @@ def GzipWriteFile(block_iter, filename, size=200 * 1024 * 1024, gzipped=True):
 
 
 def PlainWriteFile(block_iter, filename, size=200 * 1024 * 1024, gzipped=False):
-    u"""
+    """
     Write plain uncompressed file of given size
 
     This is like the earlier GPGWriteFile except it writes a gzipped
@@ -477,22 +485,30 @@ def PlainWriteFile(block_iter, filename, size=200 * 1024 * 1024, gzipped=False):
 
 
 def get_hash(hash, path, hex=1):  # pylint: disable=redefined-builtin
-    u"""
+    """
     Return hash of path
 
     hash should be "MD5" or "SHA1".  The output will be in hexadecimal
     form if hex is true, and in text (base64) otherwise.
     """
     # assert path.isreg()
-    fp = path.open(u"rb")
-    if hash == u"SHA1":
-        hash_obj = sha1()
-    elif hash == u"MD5":
-        hash_obj = md5()
+    fp = path.open("rb")
+    if hash == "SHA1":
+        # TODO: Remove when py38 goes EOL
+        if sys.version_info[:2] == (3, 8):
+            hash_obj = sha1()
+        else:
+            hash_obj = sha1(usedforsecurity=False)
+    elif hash == "MD5":
+        # TODO: Remove when py38 goes EOL
+        if sys.version_info[:2] == (3, 8):
+            hash_obj = md5()
+        else:
+            hash_obj = md5(usedforsecurity=False)
     else:
-        assert 0, u"Unknown hash %s" % (hash,)
+        assert 0, f"Unknown hash {hash}"
 
-    while 1:
+    while True:
         buf = fp.read(blocksize)
         if not buf:
             break
